@@ -3,7 +3,6 @@
 #' `sdsm` extracts the backbone of a bipartite projection using the Stochastic Degree Sequence Model.
 #'
 #' @param B An unweighted bipartite graph, as: (1) an incidence matrix in the form of a matrix or sparse \code{\link{Matrix}}; (2) an edgelist in the form of a two-column dataframe; (3) an \code{\link{igraph}} object.
-#'     Any rows and columns of the associated bipartite matrix that contain only zeros are automatically removed before computations.
 #' @param alpha real: significance level of hypothesis test(s)
 #' @param signed boolean: TRUE for a signed backbone, FALSE for a binary backbone (see details)
 #' @param mtc string: type of Multiple Test Correction to be applied; can be any method allowed by \code{\link{p.adjust}}.
@@ -28,10 +27,10 @@
 #' @return
 #' If `alpha` != NULL: Binary or signed backbone graph of class `class`.
 #'
-#' If `alpha` == NULL: An S3 backbone object containing three matrices (the weighted graph, edges' upper-tail p-values,
-#'    edges' lower-tail p-values), and a string indicating the null model used to compute p-values, from which a backbone
-#'    can subsequently be extracted using [backbone.extract()]. The `signed`, `mtc`, `class`, and `narrative` parameters
-#'    are ignored.
+#' If `alpha` == NULL: An S3 backbone object containing (1) the weighted graph as a matrix, (2) upper-tail p-values as a
+#'    matrix, (3, if `signed = TRUE`) lower-tail p-values as a matrix, (4, if present) node attributes as a dataframe, and
+#'    (5) several properties of the original graph and backbone model, from which a backbone can subsequently be extracted
+#'    using [backbone.extract()].
 #'
 #' @references package: {Neal, Z. P. (2022). backbone: An R Package to Extract Network Backbones. *PLOS ONE, 17*, e0269137. \doi{10.1371/journal.pone.0269137}}
 #' @references sdsm: {Neal, Z. P. (2014). The backbone of bipartite projections: Inferring relationships from co-authorship, co-sponsorship, co-attendance, and other co-behaviors. *Social Networks, 39*, 84-97. \doi{10.1016/j.socnet.2014.06.001}}
@@ -80,33 +79,54 @@ sdsm <- function(B, alpha = 0.05, signed = FALSE, mtc = "none", class = "origina
   #### Compute Probabilities for SDSM ####
   prob.mat <- bicm(B,...)
 
-  #### Compute p-values ####
-  Pupper <- matrix(0, nrow(P), ncol(P))
-  Plower <- matrix(0, nrow(P), ncol(P))
-  for (col in 1:ncol(P)) {  #Loop over lower triangle
-    for (row in col:nrow(P)) {
-      probs <- prob.mat[row,]*prob.mat[col,]
-      Plower[row,col] <- pb(P[row,col], probs)
-      Pupper[row,col] <- pb(P[row,col]-1, probs, lower = FALSE)
+  #### Compute p-values (for unsigned backbone, ignore lower-tail p-values) ####
+  if (!signed) {
+    Pupper <- matrix(1, nrow(P), ncol(P))  #Set upper-tail p-value to 1 initially
+    for (col in 1:ncol(P)) {  #Loop over lower triangle
+      for (row in col:nrow(P)) {
+        if (P[row,col] != 0) {  #Compute and update the upper-tail p-value only if the edge has non-zero weight
+          pvalues <- pb(P[row,col], prob.mat[row,]*prob.mat[col,], lowertail = FALSE)
+          Pupper[row,col] <- pvalues[2]
+        }
+      }
     }
+    Pupper[upper.tri(Pupper)] <- t(Pupper)[upper.tri(Pupper)]  #Add upper triangle
+  }
+
+  #### Compute p-values (for signed backbone) ####
+  if (signed) {
+    Pupper <- matrix(0, nrow(P), ncol(P))
+    Plower <- matrix(0, nrow(P), ncol(P))
+    for (col in 1:ncol(P)) {  #Loop over lower triangle
+      for (row in col:nrow(P)) {
+        pvalues <- pb(P[row,col], prob.mat[row,]*prob.mat[col,])
+        Plower[row,col] <- pvalues[1]
+        Pupper[row,col] <- pvalues[2]
+      }
+    }
+    Pupper[upper.tri(Pupper)] <- t(Pupper)[upper.tri(Pupper)]  #Add upper triangles
+    Plower[upper.tri(Plower)] <- t(Plower)[upper.tri(Plower)]
   }
-  Pupper[upper.tri(Pupper)] <- t(Pupper)[upper.tri(Pupper)]  #Add upper triangles
-  Plower[upper.tri(Plower)] <- t(Plower)[upper.tri(Plower)]
 
   #### Assemble backbone object ####
-  bb <- list(G = P, Pupper = Pupper, Plower = Plower, model = "sdsm")
+  bb <- list(G = P,  #Preliminary backbone object
+             Pupper = Pupper,
+             model = "sdsm",
+             agents = nrow(B),
+             artifacts = ncol(B),
+             weighted = FALSE,
+             bipartite = TRUE,
+             symmetric = TRUE,
+             class = class,
+             trials = NULL)
+  if (signed) {bb <- append(bb, list(Plower = Plower))}  #Add lower-tail values, if requested
+  if (!is.null(attribs)) {bb <- append(bb, list(attribs = attribs))}  #Add node attributes, if present
   class(bb) <- "backbone"
 
   #### Return result ####
   if (is.null(alpha)) {return(bb)}  #Return backbone object if `alpha` is not specified
   if (!is.null(alpha)) {            #Otherwise, return extracted backbone (and show narrative text if requested)
-    backbone <- backbone.extract(bb, alpha = alpha, signed = signed, mtc = mtc, class = "matrix")
-    reduced_edges <- round(((sum(P!=0)-nrow(P)) - sum(backbone!=0)) / (sum(P!=0)-nrow(P)),3)*100  #Percent decrease in number of edges
-    reduced_nodes <- round((max(sum(rowSums(P)!=0),sum(colSums(P)!=0)) - max(sum(rowSums(backbone)!=0),sum(colSums(backbone)!=0))) / max(sum(rowSums(P)!=0),sum(colSums(P)!=0)),3) * 100  #Percent decrease in number of connected nodes
-    if (narrative == TRUE) {write.narrative(agents = nrow(B), artifacts = ncol(B), weighted = FALSE, bipartite = TRUE, symmetric = TRUE,
-                                            signed = signed, mtc = mtc, alpha = alpha, s = NULL, ut = NULL, lt = NULL, trials = NULL, model = "sdsm",
-                                            reduced_edges = reduced_edges, reduced_nodes = reduced_nodes)}
-    backbone <- frommatrix(backbone, attribs, convert = class)
+    backbone <- backbone.extract(bb, alpha = alpha, signed = signed, mtc = mtc, class = class, narrative = narrative)
     return(backbone)
   }
 }
